from django import forms
from rest_framework.views import APIView
from django.shortcuts import render, redirect
from rest_framework.decorators import api_view

from django.template import Library
from django.http import HttpResponse
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from .models import User, Profile
from django.core.exceptions import PermissionDenied
from position.models import Position
from sales.models import Sales
from company.models import Company
from cuser.middleware import CuserMiddleware
import re
from rest_framework.response import Response
from django.utils.safestring import mark_safe
from datetime import datetime, timedelta
from rest_framework.authtoken.models import Token
from django.utils import timezone
from django.conf import settings


#this return left time
def expires_in(token):
    time_elapsed = timezone.now().replace(tzinfo=None) - token.created.replace(tzinfo=None)
    left_time = timedelta(seconds=settings.TOKEN_EXPIRED_AFTER_SECONDS) - time_elapsed
    return left_time

# token checker if token expired or not
def is_token_expired(token):
    return expires_in(token) < timedelta(seconds = 0)


def token_expire_handler(token):
    # if token is expired new token will be established
    # If token is expired then it will be removed
    # and new one with different key will be created
    is_expired = is_token_expired(token)
    if is_expired:
        token.delete()
        return True
    else:
        return False


@api_view(['POST'])
def user_login(request):
    if request.method == 'POST':
        username = request.data.get('username', '')
        password = request.data.get('password', '')
        if not username:
            return Response({"message": "lack of username", "status_code": 400})
        if not password:
            return Response({"message": "lack of password", "status_code": 400})
        user = authenticate(request, username=username, password=password)
        if not user:
            return Response({"message": "wrong username or password", "status_code": 400})
        else:
            token = Token.objects.filter(user=user)
            if token:
                token.delete()
            token = Token.objects.create(user=user)
            unit_objs = []
            for unit in user.unit.all():
                unit_objs.append({"num": unit.unit_num,
                                  "name": unit.unit_name,
                                  "type": unit.unit_type})
            return Response({"message": "login success",
                             "status_code": 200,
                             "token": token.key,
                             "full_name": user.full_name,
                             "unit": unit_objs})

def get_sales_unit_detail():
    result = []
    for this_sales in Sales.objects.filter(state="在職").exclude(sales_unit__unit_name="看不到"):
        result.append(
            {
                "UnitName": this_sales.sales_unit.unit_name,
                "UnitID": this_sales.sales_unit.id,
                "SalesID": this_sales.id,
                "SalesName": this_sales.full_name
            }
        )
    return result

   
@api_view(['GET'])
def get_sales_data(request):
    if request.method == 'GET':
            return Response(get_sales_unit_detail())

@api_view(['POST'])
def list_sales_by_unit(request):
    if request.method == 'POST':
        token = request.data.get('token', '')
        num = request.data.get('num', '')
        if not token:
            return Response({"message": "lack of token", "status_code": 400})
        if not num:
            return Response({"message": "lack of num", "status_code": 400})
        token = Token.objects.filter(key=token)
        if not token:
            return Response({"message": "useless token", "status_code": 400})
        token = token[0]
        if token_expire_handler(token):
            return Response({"message": "token expired, login again", "status_code": 400})
        token.created = datetime.now()
        token.save()
        user = token.user
        unit = user.unit.filter(unit_num=num)
        if not unit:
            return Response({"message": "this user not in this unit", "status_code": 400})
        else:
            unit = unit[0]
            return Response({"message": "list unit sales success",
                             "status_code": 200,
                             "unit_num": unit.unit_num,
                             "unit_name": unit.unit_name,
                             "sales": [{"id": sales.id,
                                        "name": sales.full_name} for sales in unit.sales_set.filter()]})


class ChangePasswordForm(forms.Form):
    error_css_class = 'error'
    originpswd = forms.CharField(
			label="舊密碼",
			widget=forms.PasswordInput(
				attrs={
					"class": "form-control",
					"placeholder": "請輸入舊密碼",
					}
			),
		)
    pswd = forms.CharField(
			label="新密碼",
			widget=forms.PasswordInput(
				attrs={
					"class": "form-control",
					"placeholder": "請輸入新密碼",
					}
			)
		)
    pswd2 = forms.CharField(
			label="重複新密碼",
			widget=forms.PasswordInput(
				attrs={
					"class": "form-control",
					"placeholder": "請重複新密碼",
					}
			),
            help_text=mark_safe("為檢查用，請輸入與前面相同的密碼。")
		)
    def clean_originpswd(self):
        originpswd = self.cleaned_data.get('originpswd', None)
        user = CuserMiddleware.get_user()
        if user and originpswd:
            if not user.check_password(originpswd):
                raise forms.ValidationError("錯誤的原始密碼")
            else:
                return originpswd
        elif not originpswd:
            raise forms.ValidationError("請填入原始密碼")
        else:
            return originpswd

    def clean_pswd(self):
        pswd = self.cleaned_data.get('pswd', None)
        if pswd:
            if len(pswd) < 8:
                raise forms.ValidationError("密碼長度需大於8")
            special = re.search(r"[%@#$%&_=-]", pswd)
            if not special:
                raise forms.ValidationError("請至少包含一個下列特殊符號，% @ # $ % & _ = -")
            uppercase = re.search(r"[A-Z]", pswd)
            lowercase = re.search(r"[a-z]", pswd)
            digit     = re.search(r"[0-9]", pswd)
            if not uppercase:
                raise forms.ValidationError("請至少包含一個大寫英文字母")
            if not lowercase:
                raise forms.ValidationError("請至少包含一個小寫英文字母")
            if not digit:
                raise forms.ValidationError("請至少包含一個數字")
            user = CuserMiddleware.get_user()
            pro  = getattr(user, 'profile', None)
            if pro:
                p1 = pro.pswd
                p2 = pro.pswd2
                if p1 == pswd or p2 == pswd or user.check_password(pswd):
                    raise forms.ValidationError("不得與前三次的密碼相同")
            return pswd
        raise forms.ValidationError("請輸入新密碼")

    def clean_pswd2(self):
        pswd = self.cleaned_data.get('pswd', None)
        pswd2 = self.cleaned_data.get('pswd2', None)
        if pswd and pswd2 and pswd != pswd2:
            raise forms.ValidationError(
                "兩個密碼欄位不相符。",
                code='password_mismatch',
            )
        return pswd2


@login_required(login_url='/login/')
def change_password_page(request):
    message = request.GET.get('message')
    form = ChangePasswordForm(request.POST or None)
    content = {
		"form": form,
		"error":""
	}
    print(form)
    if form.is_valid():
        print(form.cleaned_data)
        user = CuserMiddleware.get_user()
        if user :
            profile = user.profile
            profile.pswd2 = profile.pswd
            profile.pswd  = form.cleaned_data.get("originpswd", None)
            profile.updated = datetime.today()
            user.set_password(form.cleaned_data.get("pswd", None))
            user.save()
            profile.save()
            return redirect("/logout/")
    return render(request, "forgotPassword.html", content)

# Create your views here.
